#include "led.h"

led_state::led_state()
{
    m_eMode = eLEDOff;
    m_iValue = 0;
    m_iTicks = 0;
    m_iRate = 0;
}

bool led_state::tick()
{
    if (m_eMode == eLEDOff) return false;
    
    if (m_eMode == eLEDBlink)
    {
        // Decrement the counter until it gets to 0
        m_iCounter--;
        if (m_iCounter < 1)
        {
            // Toggle the LED
            m_iValue = !m_iValue;
            // Reset the counter
            m_iCounter = m_iRate;
        }
    }
    
    // Decrement the ticks
    if (m_iTicks > 0)
    {
        m_iTicks--;
        if (m_iTicks < 1)
        {
            // Turn the LED Off
            m_iValue = 0;
            // Set the MODE to Off
            m_eMode = eLEDOff;
        }
    }
    
    return true;
}

void led_state::set(bool on, int ticks)
{
    // Set the ticks
    m_iTicks = ticks;
    
    if (on)
    {
        m_eMode = eLEDOn;
        m_iValue = 1;   
    }
    else
    {
        m_eMode = eLEDOff;
        m_iValue = 0;
    }
}

void led_state::blink(int rate, int state, int ticks)
{
    // Bounds check the rate
    if (rate < 1) return;
    
    // Set the rate
    m_iRate = rate;
    m_iCounter = rate;
    
    // Set the ticks
    m_iTicks = ticks;
    
    // Set the mode
    m_eMode = eLEDBlink;
    
    // Set the initial value
    m_iValue = state;
}

led::led() : led1(P0_9), led2(P0_8)
{
    led1 = 0;
    led2 = 0;
}

void led::tick()
{
    if (led_state1.tick()) led1 = led_state1.m_iValue;
    if (led_state2.tick()) led2 = led_state2.m_iValue;
}

bool led::get(int led)
{
    if (led == 1) return (led_state1.m_eMode != led_state::eLEDOff);
    if (led == 2) return (led_state2.m_eMode != led_state::eLEDOff);
    return false;
}
    
void led::set(int led, bool on, int ticks)
{
    if (led == 1)
    {
        led_state1.set(on, ticks);
        led1 = led_state1.m_iValue;
    }
    if (led == 2)
    {
        led_state2.set(on, ticks);
        led2 = led_state2.m_iValue;
    }
}

void led::blink(int led, int rate, int state, int ticks)
{
    if (led == 1)
    {
        led_state1.blink(rate, state, ticks);
        led1 = led_state1.m_iValue;
    }
    if (led == 2)
    {
        led_state2.blink(rate, state, ticks);
        led2 = led_state2.m_iValue;
    }
}
